import { getOrderBySubOrderNumber, saveOrder } from '@/api/order'
import {
  OrderData,
  ProductList,
  ShipmentForm,
  ShipmentOrderRes,
} from '@/types/api/order'
import { useValue } from '@/utils/hooks/useValue'
import { showError } from '@/utils/ui'
import { ref } from 'vue'
const [orderList, resetOrderList] = useValue<OrderData[]>([])
const [shipmentForm, resetShipmentForm] = useValue<ShipmentForm>({
  shippingWay: '',
  carriageName: '',
  logisticsTracking: '',
  carriageAmount: '',
  namespace: '',
})
export default function useShipment(callback?: () => void) {
  const shipmentFormRef = ref()
  const productionOrderNumber = ref('')
  const shipmentVisible = ref(false)
  const isLock = ref(false)
  const inputRef = ref()
  const currentRow = ref<ProductList>()
  const shipmentLoading = ref(false)
  const shipmentOrderRef = ref()
  const searchShipmentByOrderNumber = async () => {
    const code = productionOrderNumber.value
    shipmentVisible.value = true
    if (!code) {
      isLock.value = false
      inputRef.value.focus()
      return ElMessage({
        message: '请录入生产单号',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
    }
    let rowData
    const code1 = code?.split('_')[0]
    for (const item of orderList.value) {
      rowData = item.productList?.find((jj) => jj.subOrderNumber === code1)
      if (rowData) break
    }
    if (rowData) {
      currentRow.value = rowData
      const unShipmentNum =
        (rowData.num || 0) -
        (rowData.shipmentNum || 0) +
        (rowData.notPassNum || 0)
      if (unShipmentNum > (rowData.count || 0)) {
        rowData.count = unShipmentNum
        if (+rowData.count > 1) {
          ElMessageBox.confirm(`该产品未发数为${rowData.count}件`, '重要提示', {
            confirmButtonText: '确定',
            type: 'warning',
          }).catch(() => {})
        }
      } else {
        ElMessage({
          message: '拣货数不能大于未发数',
          type: 'warning',
          offset: window.innerHeight / 2,
        })
      }
      inputRef.value.focus()
      productionOrderNumber.value = ''
      isLock.value = false

      if (shipmentOrderRef.value) {
        const rowEl = shipmentOrderRef.value.querySelector(
          `div[data-id="${rowData.id}"]`,
        )
        if (!rowEl) return

        rowEl.scrollIntoView()
      }
    } else {
      getPackingData(code)
    }
  }
  const getPackingData = async (code: string) => {
    try {
      shipmentLoading.value = true
      const res = await getOrderBySubOrderNumber(code)
      if (res.data) {
        const code1 = code?.split('_')[0]
        for (const item of res.data.productList || []) {
          item.count = 0
          if (item.subOrderNumber === code1) {
            item.count =
              (item.num || 0) -
              ((item.shipmentNum || 0) - (item.notPassNum || 0))
            if (+item.count > 1) {
              ElMessageBox.confirm(
                `该产品未发数为${item.count}件`,
                '重要提示',
                {
                  confirmButtonText: '确定',
                  type: 'warning',
                },
              ).catch(() => {})
            }
            currentRow.value = item
            if (shipmentOrderRef.value) {
              const rowEl = shipmentOrderRef.value.querySelector(
                `div[data-id="${item.id}"]`,
              )
              if (!rowEl) return

              rowEl.scrollIntoView()
            }
          }
        }
        const index = orderList.value.findIndex(
          (item) => item.id === res.data.id,
        )
        if (index === -1) {
          orderList.value.unshift(res.data)
        }
      }
      inputRef.value.focus()
      productionOrderNumber.value = ''
      isLock.value = false
    } catch (e) {
      productionOrderNumber.value = ''
      isLock.value = false
      inputRef.value.focus()
      showError(e)
    } finally {
      shipmentLoading.value = false
    }
  }
  const saveShipment = async () => {
    if (orderList.value.length === 0) {
      return ElMessage({
        message: '请选择订单',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
    }
    try {
      await shipmentFormRef.value?.validate()
    } catch {
      return
    }

    const isEqual = orderList.value.some(
      (item) => item.namespace === orderList.value[0].namespace,
    )
    if (!isEqual) {
      ElMessage({
        message: '请选择同一工厂的订单',
        type: 'warning',
        offset: window.innerHeight / 2,
      })
      return
    }
    const data: ShipmentOrderRes[] = []
    orderList.value.forEach((item) => {
      shipmentForm.value.namespace = item.namespace
      const order = []
      for (const jj of item.productList || []) {
        if (jj.count === 0) continue
        const subOrder = {
          erpSubOrderNumber: jj.erpSubOrderNumber,
          subOrderNumber: jj.subOrderNumber,
          sendOutQuantity: jj.count,
          version: jj.version,
        }
        order.push(subOrder)
      }
      if (order) {
        data.push(...order)
      }
    })
    try {
      const res = await saveOrder(data, shipmentForm.value)
      ElMessage.success(res.message)
      shipmentVisible.value = false
      callback && callback()
    } catch (e) {
      showError(e)
    }
  }
  const confirmDelivery = async () => {
    shipmentVisible.value = true
    productionOrderNumber.value = ''
    resetOrderList()
    resetShipmentForm()
  }
  const onShipmentDialogOpened = () => {
    inputRef.value?.focus()
    shipmentFormRef.value?.clearValidate()
  }
  return {
    shipmentFormRef,
    shipmentOrderRef,
    productionOrderNumber,
    shipmentVisible,
    inputRef,
    shipmentForm,
    currentRow,
    shipmentLoading,
    orderList,
    searchShipmentByOrderNumber,
    saveShipment,
    confirmDelivery,
    onShipmentDialogOpened,
  }
}
